class _MatchErrorMixin:
    match = ''
    text = None

    __subclasses = []

    def __init_subclass__(cls, **kwargs):
        super(_MatchErrorMixin, cls).__init_subclass__(**kwargs)
        # cls.match = cls.match.lower() if cls.match else ''
        if not hasattr(cls, f"_{cls.__name__}__group"):
            cls.__subclasses.append(cls)

    @classmethod
    def check(cls, message) -> bool:
        """
        Compare pattern with message
        :param message: always must be in lowercase
        :return: bool
        """
        return cls.match.lower() in message

    @classmethod
    def get_text(cls, message) -> str:
        """
        Get text from message
        :param message: always must be in lowercase
        :return: str
        """
        return cls.text or message

    @classmethod
    def detect(cls, description):
        description = description.lower()
        for err in cls.__subclasses:
            if err is cls:
                continue
            if err.check(description):
                raise err(err.text or description)
        raise cls(description)


class APIError(Exception, _MatchErrorMixin):
    pass



from abc import ABC
from typing import Optional, List

from ..core.abc import BaseAPIClient
from ..core.methods import Methods
from ..types.sms import Balance, NumberStats, OrderNumber, StateInfo, OrderManaged


class OnlineSMS(BaseAPIClient, ABC):
    BASE_URL = 'https://onlinesim.ru/api/'

    def __init__(self, api_key: str):
        super().__init__(api_key)

    async def get_services(self, country: Optional[str] = None) -> NumberStats:
        """
        Get Number Stats
        :param country: Country (If not provided will return all the countries)
        :return: NumberStats
        """
        params = {"country": country}
        result = await self._send_request(Methods.GET_NUMBERS_STATS, params)
        return NumberStats(**result)

    async def order_number(self, service: str, region: Optional[int] = None, country: Optional[int] = None,
                           reject: Optional[List[int]] = None, extension: Optional[int] = None,
                           number: Optional[bool] = None, dev_id: Optional[int] = None) -> OrderNumber:
        """
        Order Number
        :param service: Service ID
        :param region: Region (77 - Moscow for example)
        :param country: Country ID (default=7)
        :param reject: Reject numbers mask ([911, 912] for example)
        :param extension: Extension (default=0)
        :param number: Return Number (default=False)
        :param dev_id: Developer ID
        :return:
        """
        params = {"service": service, "region": region, "country": country, "reject": reject, "extension": extension,
                  "dev_id": dev_id, "number": str(number) if number else None}
        result = await self._send_request(Methods.GET_NUM, params)
        return OrderNumber(**result)

    async def get_order_info(self, operation_id: int, get_full_message: Optional[bool] = False,
                             form: Optional[int] = None, order_by: Optional[str] = None,
                             msg_list: Optional[int] = None, clean: Optional[int] = None) -> StateInfo:
        """
        Get Order Info
        :param clean: Do not show messages on a circle (?).
        :param msg_list: Type of message list, 1 - the list, 0 - the active message
        :param order_by: ascending/descending sorting asc/desc. Default value - asc
        :param form: type of reception, 1 - online reception, 2 - repeated reception, 3 - delayed reception.
        :param get_full_message: Get Full Message. If True, will return the full message.
        :param operation_id: Operation ID
        :return: Order Info
        """
        params = {"tzid": operation_id, "message_to_code": 0 if get_full_message else 1,
                  "form": form, "order_by": order_by, "msg_list": msg_list, "clean": clean}
        result = await self._send_request(Methods.GET_STATE, params)
        corrected_json = {"response": "1", "orders": result}
        return StateInfo(**corrected_json)

    async def finish_order(self, operation_id: int, ban: Optional[bool] = False) -> OrderManaged:
        """
        Finish Order
        :param ban: Block Number (Only for Chinese numbers)
        :param operation_id: Operation ID
        :return: None
        """
        params = {"tzid": operation_id, "ban": 1 if ban else 0}
        result = await self._send_request(Methods.SET_OPERATION_OK, params)
        return OrderManaged(**result)

    async def revise_order(self, operation_id: int) -> OrderManaged:
        """
        Revise Order
        :param operation_id: Operation ID
        :return: None
        """
        params = {"tzid": operation_id}
        result = await self._send_request(Methods.SET_OPERATION_REVISE, params)
        return OrderManaged(**result)



from enum import Enum


class Methods(Enum):
    GET_BALANCE = 'getBalance'
    GET_NUMBERS_STATS = 'getNumbersStats'
    GET_NUM = 'getNum'
    GET_STATE = 'getState'
    SET_OPERATION_REVISE = 'setOperationRevise'
    SET_OPERATION_OK = 'setOperationOk'


class RentMethods(Enum):
    TARIFFS_RENT = 'tariffsRent'
    GET_RENT_NUM = 'getRentNum'
    GET_RENT_STATE = 'getRentState'
    EXTEND_RENT_STATE = 'extendRentState'
    CLOSE_RENT_NUM = 'closeRentNum'


class ProxyMethods(Enum):
    pass

__all__ = ['Methods', 'RentMethods', 'ProxyMethods']


from .balance import Balance
from .manage_order import OrderManaged
from .number_stats import NumberStats, ServiceInfo
from .order_number import OrderNumber
from .state import StateInfo, OrderState

__all__ = ['Balance', 'NumberStats', 'ServiceInfo', 'OrderNumber', 'StateInfo', 'OrderState', 'OrderManaged']



import abc
from typing import Optional, Union, List

from aiohttp import ClientSession

from ... import exceptions
from ..methods import Methods, RentMethods
from ...types import Balance


class BaseAPIClient(abc.ABC):
    BASE_URL = 'https://onlinesim.ru/api/'

    def __init__(self, api_key: str):
        self.api_key = api_key

    async def get_balance(self) -> Balance:
        """
        Get Account Balance Method
        :return: Balance
        """
        result = await self._send_request(Methods.GET_BALANCE)
        return Balance(**result)

    async def _send_request(self, method: Union[Methods, RentMethods], params: Optional[dict] = None) -> dict:
        """
        Send request to the API
        :param method: API Method
        :param params: Parameters
        :return: JSON
        """
        async with ClientSession() as session:
            params = self._delete_none(params or {})
            params['apikey'] = self.api_key
            request_url = self._get_request_url(method)
            response = await session.get(request_url, params=params)
            json = await response.json()
            if isinstance(json, dict):
                if json.get("response", None) and str(json['response']) != '1':
                    exceptions.APIError.detect(json['response'])
                elif json.get('original', None) and json['original']['response'] != '1':
                    exceptions.APIError.detect(json['original']['response'])
            return json

    def _get_request_url(self, method: Union[Methods, RentMethods]) -> str:
        if self.BASE_URL.endswith('rent/') and method == Methods.GET_BALANCE:
            return f"{self.BASE_URL.replace('rent/', '')}{method.value}.php"
        return f'{self.BASE_URL}{method.value}.php'

    def _delete_none(self, _dict: dict) -> dict:
        """Delete None values recursively from all of the dictionaries"""
        for key, value in list(_dict.items()):
            if isinstance(value, dict):
                self._delete_none(value)
            elif value is None:
                del _dict[key]
            elif isinstance(value, list):
                for v_i in value:
                    if isinstance(v_i, dict):
                        self._delete_none(v_i)

        return _dict

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        return None



from typing import Optional

from pydantic import BaseModel, Field


class OrderNumber(BaseModel):
    response: str
    operation_id: int = Field(..., alias='tzid')
    number: Optional[str]
    country: Optional[str]



from pyonlinesim.exceptions.base import APIError


class ExceededConcurrentOperations(APIError):
    match = 'exceeded_concurrent_operations'
    text = 'Exceeded concurrent operations for your account'


class NoNumber(APIError):
    match = 'no_number'
    text = 'No available numbers for this service'


class TimeIntervalError(APIError):
    match = 'time_interval_error'
    text = 'Delayed forwarding is not available at this moment'


class IntervalConcurrentRequestsError(APIError):
    match = 'interval_concurrent_requests_error'
    text = 'Exceeded concurrent requests. Try later'


class NoDefferForward(APIError):
    match = 'no_forward_for_deffer'
    text = 'No forward for deffer'


class NoNumberForForward(APIError):
    match = 'no_number_for_forward'
    text = 'No available numbers for forwarding'


class ForwardLengthError(APIError):
    match = 'error_length_number_for_forward'
    text = 'Forwarding number has wrong length'


class DuplicateError(APIError):
    match = 'duplicate_operation'
    text = 'Duplicate operation'


class NotEnoughFunds(APIError):
    match = 'warning_low_balance'
    text = 'Not enough funds'


class WrongOperationID(APIError):
    match = 'error_wrong_tzid'
    text = 'Wrong operation ID'


class CantFinishOrder(APIError):
    match = 'no_complete_tzid'
    text = 'Unable to finish order'


from typing import List

from pydantic import BaseModel, validator


class ServiceInfo(BaseModel):
    id: int
    count: int
    popular: bool
    code: int
    price: float
    service: str
    slug: str


class NumberStats(BaseModel):
    name: str
    position: int
    code: int
    other: float
    new: bool
    enabled: bool
    services: List[ServiceInfo]

    @validator('services', pre=True)
    def iter_to_list(cls, v):
        return [v[key] for key in list(v)]



from ..base import APIError


class UndefinedCountry(APIError):
    match = 'undefined_country'
    text = 'Undefined country'


class UndefinedDays(APIError):
    match = 'undefined_days'
    text = 'Undefined days'



from typing import List, Optional

from pydantic import BaseModel, Field, validator


class NumberData(BaseModel):
    price: int
    amount: int


class Numbers(BaseModel):
    one_day: Optional[NumberData] = Field(None, alias='1')
    three_days: Optional[NumberData] = Field(None, alias='3')
    seven_days: Optional[NumberData] = Field(None, alias='7')
    fifteen_days: Optional[NumberData] = Field(None, alias='15')
    thirty_days: Optional[NumberData] = Field(None, alias='30')


class RentTariff(BaseModel):
    id: int = Field(..., alias='code')
    is_enabled: bool = Field(..., alias='enabled')
    country_name: str = Field(..., alias='name')
    is_new: bool = Field(..., alias='new')
    position: int
    numbers: Numbers = Field(..., alias='numbers')
    extend_price: int = Field(..., alias='extend')
    confirm: Optional[bool]



from datetime import datetime
from enum import Enum
from typing import List, Optional

from pydantic import BaseModel, Field, validator


class RentStatus(int, Enum):
    WAITING = 0
    CONFIRMED = 1


class RentMessage(BaseModel):
    id: int
    code: str
    created_at: datetime
    service: str
    text: str


class RentMessages(BaseModel):
    current_page: int
    data: Optional[List[RentMessage]]
    first_page: int = Field(None, alias='from')
    last_page: int
    per_page: int
    pages_amount: int = Field(None, alias='to')
    total_messages: int = Field(None, alias='total')


class ExtendRent(BaseModel):
    one_day: Optional[int] = Field(None, alias='1')
    seven_days: Optional[int] = Field(None, alias='7')
    fifteen_days: Optional[int] = Field(None, alias='15')
    thirty_days: Optional[int] = Field(None, alias='30')


class RentNumber(BaseModel):
    status: RentStatus
    messages: Optional[RentMessages] = Field(None, alias='messages')
    country: int
    rent: int
    extension: int
    checked_time: datetime
    sum: float
    number: str
    operation_id: int = Field(..., alias='tzid')
    extend: Optional[ExtendRent] = Field(None, alias='extend')
    checked: bool
    reload: int
    day_extend: int
    m_ext: bool
    freeze: bool

    @validator('messages', pre=True)
    def validate_messages(cls, v):
        return None if v == [] else RentMessages(**v)

    @validator('extend', pre=True)
    def validate_extend(cls, v):
        return None if v == [] else ExtendRent(**v)



from .rent.order import RentStatus, RentNumber, RentMessage, RentMessages, ExtendRent
from .rent.tarrifs import Numbers, NumberData, RentTariff
from .sms.balance import Balance
from .sms.manage_order import OrderManaged
from .sms.number_stats import NumberStats, ServiceInfo
from .sms.order_number import OrderNumber
from .sms.state import StateInfo, OrderState

__all__ = ['Balance', 'NumberStats', 'ServiceInfo', 'OrderNumber', 'StateInfo', 'OrderState', 'OrderManaged',
           'RentStatus', 'RentNumber', 'RentMessage', "RentMessages", 'ExtendRent', 'Numbers',
           'NumberData', 'RentTariff']



from typing import List, Optional

from pydantic import BaseModel, Field


class OrderState(BaseModel):
    response: str
    operation_id: int = Field(..., alias='tzid')
    service: str
    number: str
    time_left: int = Field(..., alias='time')
    form: str
    country: int
    sum: Optional[int]
    forward_status: Optional[int]
    forward_number: Optional[str]


class StateInfo(BaseModel):
    response: Optional[str]
    orders: List[OrderState]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyonlinesim',
 'pyonlinesim.clients',
 'pyonlinesim.core',
 'pyonlinesim.core.abc',
 'pyonlinesim.core.methods',
 'pyonlinesim.exceptions',
 'pyonlinesim.exceptions.rent',
 'pyonlinesim.exceptions.sms',
 'pyonlinesim.types',
 'pyonlinesim.types.rent',
 'pyonlinesim.types.sms']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.3', 'pydantic>=1.9.2,<2.0.0']

setup_kwargs = {
    'name': 'pyonlinesim',
    'version': '1.1.1',
    'description': 'Asynchronous wrapper to interact with onlinesim.ru API',
    'long_description': '[![Downloads](https://pepy.tech/badge/pyonlinesim)](https://pepy.tech/project/pyonlinesim)\n[![Downloads](https://pepy.tech/badge/pyonlinesim/month)](https://pepy.tech/project/pyonlinesim)\n[![Downloads](https://pepy.tech/badge/pyonlinesim/week)](https://pepy.tech/project/pyonlinesim)\n[![Code Quality Score](https://api.codiga.io/project/34377/score/svg)](https://api.codiga.io/project/34377/score/svg)\n[![Code Grade](https://api.codiga.io/project/34377/status/svg)](https://api.codiga.io/project/34377/status/svg)\n\n## 🔗 Links\n* 🎓 **Documentation:** [*CLICK*](https://pyonlinesim.readthedocs.io/en/latest/)\n* 🖱️ **Developer contacts:** [![Dev-Telegram](https://img.shields.io/badge/Telegram-blue.svg?style=flat-square&logo=telegram)](https://t.me/marple_tech)\n## 🐦 Dependencies  \n\n| Library  |                       Description                       |\n|:--------:|:-------------------------------------------------------:|\n| aiohttp  | Asynchronous HTTP Client/Server for asyncio and Python. |\n| pydantic |                   JSON Data Validator                   |\n\n---\n',
    'author': 'Marple',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/marple-git/pyonlinesim',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from .authentication import AccountBlocked, WrongAPIKey, NoAPIKey, NoServiceProvided, \
    RequestNotFound, APIAccessDisabled, APIAccessIP, NoCountry, TryAgainLater
from .base import APIError
from .sms.order import ExceededConcurrentOperations, NoNumber, TimeIntervalError, \
    NoDefferForward, NoNumberForForward, ForwardLengthError, DuplicateError, NotEnoughFunds, \
    IntervalConcurrentRequestsError


__all__ = ['APIError', 'AccountBlocked', 'WrongAPIKey',
           'NoAPIKey', 'NoServiceProvided', 'RequestNotFound',
           'APIAccessDisabled', 'APIAccessIP', 'NotEnoughFunds',
           'ExceededConcurrentOperations', 'NoNumber', 'TimeIntervalError',
           'IntervalConcurrentRequestsError', 'NoDefferForward', 'NoNumberForForward',
           'ForwardLengthError', 'DuplicateError', 'TryAgainLater', 'NoCountry']



from pyonlinesim.exceptions.base import APIError


class OperationsNotFound(APIError):
    match = 'error_no_operations'
    text = 'Operations not found'


class IdentificationRequired(APIError):
    match = 'account_identification_required'
    text = 'Account identification required'



from .base import APIError


class AccountBlocked(APIError):
    match = 'account_blocked'
    text = 'Your account is blocked'


class WrongAPIKey(APIError):
    match = 'error_wrong_key'
    text = 'Wrong API Key'


class NoAPIKey(APIError):
    match = 'error_no_key'
    text = 'No API Key provided'


class NoServiceProvided(APIError):
    match = 'error_no_service'
    text = 'No service provided'


class RequestNotFound(APIError):
    match = 'request_not_found'
    text = 'Request not found'


class APIAccessDisabled(APIError):
    match = 'api_access_disabled'
    text = 'API access disabled'


class APIAccessIP(APIError):
    match = 'api_access_ip'
    text = 'Allow connections from your IP Address'


class TryAgainLater(APIError):
    match = 'try_again_later'
    text = 'Try again later'


class NoCountry(APIError):
    match = 'no_country'
    text = 'No country provided'



from abc import ABC
from typing import Optional, List

from ..core.abc import BaseAPIClient
from ..core.methods import RentMethods
from ..types import OrderManaged
from ..types.rent import RentTariff, RentNumber


class OnlineRent(BaseAPIClient, ABC):
    BASE_URL = 'https://onlinesim.ru/api/rent/'

    def __init__(self, api_key: str):
        super().__init__(api_key)

    async def get_tariffs(self, country: Optional[int] = None) -> List[RentTariff]:
        """
        Get rent tariffs
        :param country: Country ID
        :return: List[RentTariff]
        """
        params = {'country': country}
        results = await self._send_request(RentMethods.TARIFFS_RENT, params=params)
        return [self._modify_tariffs_json(v) for v in results.values()]

    async def rent_number(self, country: int, days: int, auto_renewal: Optional[bool] = False,
                          pagination: Optional[bool] = False) -> RentNumber:
        """
        Rent a number
        :param country: Country ID
        :param days: Rent length in days
        :param auto_renewal: Auto-extend the rent
        :param pagination: Paginate messages
        :return: RentNumber
        """
        params = {'country': country, 'days': days, 'extension': str(auto_renewal), 'pagination': str(pagination)}
        result = await self._send_request(RentMethods.GET_RENT_NUM, params=params)
        json_modified = self._modify_rent_json(result['item'])
        return RentNumber(**json_modified)

    async def get_rent_info(self, operation_id: int, pagination: Optional[bool] = False) -> RentNumber:
        """
        Get rent info
        :param operation_id: Operation ID
        :param pagination: Paginate messages
        :return:
        """
        params = {'operation_id': operation_id, 'pagination': str(pagination)}
        result = await self._send_request(RentMethods.GET_RENT_STATE, params=params)
        json_modified = self._modify_rent_json(result['list'][0])
        return RentNumber(**json_modified)

    async def extend_rent(self, operation_id: int, days: int) -> RentNumber:
        """
        Extend number rent
        :param operation_id: Operation ID
        :param days: Extend for days
        :return:
        """
        params = {'operation_id': operation_id, 'days': days}
        result = await self._send_request(RentMethods.EXTEND_RENT_STATE, params=params)
        json_modified = self._modify_rent_json(result['list'][0])
        return RentNumber(**json_modified)

    async def finish_rent(self, operation_id: int) -> OrderManaged:
        """
        Finish rent
        :param operation_id: Operation ID
        :return:
        """
        params = {'operation_id': operation_id}
        result = await self._send_request(RentMethods.CLOSE_RENT_NUM, params=params)
        return OrderManaged(**result)

    @staticmethod
    def _modify_tariffs_json(json: dict) -> RentTariff:
        numbers_json = {"numbers": {}}
        for k, v in json['days'].items():
            numbers_json["numbers"][k] = {'amount': json['count'][k], 'price': json['days'][k]}
        json |= numbers_json
        json.pop('count')
        json.pop('days')
        return RentTariff(**json)

    @staticmethod
    def _modify_rent_json(json: dict):
        new_json = {'time_left': {}}
        new_json['time_left']['days'] = json['days']
        new_json['time_left']['hours'] = json['hours']
        new_json['time_left']['minutes'] = json['time']
        json |= new_json
        json.pop('days')
        json.pop('hours')
        json.pop('time')
        return json
